@@ -12,9 +12,7 @@ from pkg_resources import resource_filename
 from bs4 import BeautifulSoup
 import pyvo
 
-
-from six.moves.urllib_parse import urljoin
-import six
+from urllib.parse import urljoin
 from astropy.table import Table, Column, vstack
 from astroquery import log
 from astropy.utils import deprecated
@@ -112,7 +110,7 @@ ALMA_FORM_KEYS = {
     },
     'Energy': {
         'Frequency (GHz)': ['frequency', 'frequency', _gen_numeric_sql],
-        'Bandwidth (GHz)': ['bandwidth', 'bandwidth', _gen_numeric_sql],
+        'Bandwidth (Hz)': ['bandwidth', 'bandwidth', _gen_numeric_sql],
         'Spectral resolution (KHz)': ['spectral_resolution',
                                       'em_resolution', _gen_spec_res_sql],
         'Band': ['band_list', 'band_list', _gen_band_list_sql]
@@ -545,18 +543,19 @@ class AlmaClass(QueryWithLogin):
 
         """
         Return information about the data associated with ALMA uid(s)
+
         Parameters
         ----------
-        uids: list or str
+        uids : list or str
             A list of valid UIDs or a single UID.
             UIDs should have the form: 'uid://A002/X391d0b/X7b'
-        expand_tarfiles: bool
+        expand_tarfiles : bool
             False to return information on the tarfiles packages containing
             the data or True to return information about individual files in
             these packages
-        with_auxiliary: bool
+        with_auxiliary : bool
             True to include the auxiliary packages, False otherwise
-        with_rawdata: bool
+        with_rawdata : bool
             True to include raw data, False otherwise
 
         Returns
@@ -567,7 +566,7 @@ class AlmaClass(QueryWithLogin):
         """
         if uids is None:
             raise AttributeError('UIDs required')
-        if isinstance(uids, six.string_types + (np.bytes_,)):
+        if isinstance(uids, (str, bytes)):
             uids = [uids]
         if not isinstance(uids, (list, tuple, np.ndarray)):
             raise TypeError("Datasets must be given as a list of strings.")
@@ -578,7 +577,7 @@ class AlmaClass(QueryWithLogin):
             if res.status[0] != 'OK':
                 raise Exception('ERROR {}: {}'.format(res.status[0],
                                                       res.status[1]))
-            temp = res.table
+            temp = res.to_table()
             if ASTROPY_LT_4_1:
                 # very annoying
                 for col in [x for x in temp.colnames
@@ -634,8 +633,8 @@ class AlmaClass(QueryWithLogin):
                 result = vstack([result, expanded_result], join_type='exact')
         else:
             result = result[np.logical_or(np.core.defchararray.find(
-                result['semantics'], DATALINK_SEMANTICS) == -1,
-                result['content_type'] != DATALINK_FILE_TYPE)]
+                result['semantics'].astype(str), DATALINK_SEMANTICS) == -1,
+                result['content_type'].astype(str) != DATALINK_FILE_TYPE)]
 
         return result
 
@@ -647,9 +646,11 @@ class AlmaClass(QueryWithLogin):
         query = "select distinct data_rights from ivoa.obscore where " \
                 "obs_id='{}'".format(uid)
         result = self.query_tap(query)
-        if not result or len(result.table) == 0:
+        if result:
+            tableresult = result.to_table()
+        if not result or len(tableresult) == 0:
             raise AttributeError('{} not found'.format(uid))
-        if len(result.table) == 1 and result.table[0][0] == 'Public':
+        if len(tableresult) == 1 and tableresult[0][0] == 'Public':
             return False
         return True
 
@@ -676,7 +677,7 @@ class AlmaClass(QueryWithLogin):
         return data_sizes, totalsize.to(u.GB)
 
     def download_files(self, files, savedir=None, cache=True,
-                       continuation=True, skip_unauthorized=True):
+                       continuation=True, skip_unauthorized=True,):
         """
         Given a list of file URLs, download them
 
@@ -707,49 +708,72 @@ class AlmaClass(QueryWithLogin):
         downloaded_files = []
         if savedir is None:
             savedir = self.cache_location
-        for fileLink in unique(files):
+        for file_link in unique(files):
+            log.debug("Downloading {0} to {1}".format(file_link, savedir))
             try:
-                log.debug("Downloading {0} to {1}".format(fileLink, savedir))
-                check_filename = self._request('HEAD', fileLink, auth=auth,
-                                               stream=True)
+                check_filename = self._request('HEAD', file_link, auth=auth)
                 check_filename.raise_for_status()
-                if 'text/html' in check_filename.headers['Content-Type']:
-                    raise ValueError("Bad query.  This can happen if you "
-                                     "attempt to download proprietary "
-                                     "data when not logged in")
+            except requests.HTTPError as ex:
+                if ex.response.status_code == 401:
+                    if skip_unauthorized:
+                        log.info("Access denied to {url}.  Skipping to"
+                                 " next file".format(url=file_link))
+                        continue
+                    else:
+                        raise(ex)
+
+            try:
+                filename = re.search("filename=(.*)",
+                                     check_filename.headers['Content-Disposition']).groups()[0]
+            except KeyError:
+                log.info(f"Unable to find filename for {file_link}  "
+                         "(missing Content-Disposition in header).  "
+                         "Skipping to next file.")
+                continue
+
+            if savedir is not None:
+                filename = os.path.join(savedir,
+                                        filename)
+
+            try:
+                self._download_file(file_link,
+                                    filename,
+                                    timeout=self.TIMEOUT,
+                                    auth=auth,
+                                    cache=cache,
+                                    method='GET',
+                                    head_safe=False,
+                                    continuation=continuation)
 
-                filename = self._request("GET", fileLink, save=True,
-                                         savedir=savedir,
-                                         timeout=self.TIMEOUT,
-                                         cache=cache,
-                                         auth=auth,
-                                         continuation=continuation)
                 downloaded_files.append(filename)
             except requests.HTTPError as ex:
                 if ex.response.status_code == 401:
                     if skip_unauthorized:
                         log.info("Access denied to {url}.  Skipping to"
-                                 " next file".format(url=fileLink))
+                                 " next file".format(url=file_link))
                         continue
                     else:
                         raise(ex)
                 elif ex.response.status_code == 403:
-                    log.error("Access denied to {url}".format(url=fileLink))
-                    if 'dataPortal' in fileLink and 'sso' not in fileLink:
+                    log.error("Access denied to {url}".format(url=file_link))
+                    if 'dataPortal' in file_link and 'sso' not in file_link:
                         log.error("The URL may be incorrect.  Try using "
                                   "{0} instead of {1}"
-                                  .format(fileLink.replace('dataPortal/',
-                                                           'dataPortal/sso/'),
-                                          fileLink))
+                                  .format(file_link.replace('dataPortal/',
+                                                            'dataPortal/sso/'),
+                                          file_link))
                     raise ex
                 elif ex.response.status_code == 500:
                     # empirically, this works the second time most of the time...
-                    filename = self._request("GET", fileLink, save=True,
-                                             savedir=savedir,
-                                             timeout=self.TIMEOUT,
-                                             cache=cache,
-                                             auth=auth,
-                                             continuation=continuation)
+                    self._download_file(file_link,
+                                        filename,
+                                        timeout=self.TIMEOUT,
+                                        auth=auth,
+                                        cache=cache,
+                                        method='GET',
+                                        head_safe=False,
+                                        continuation=continuation)
+
                     downloaded_files.append(filename)
                 else:
                     raise ex
@@ -782,7 +806,7 @@ class AlmaClass(QueryWithLogin):
         downloaded_files : list
             A list of the downloaded file paths
         """
-        if isinstance(uids, six.string_types + (np.bytes_,)):
+        if isinstance(uids, (str, bytes)):
             uids = [uids]
         if not isinstance(uids, (list, tuple, np.ndarray)):
             raise TypeError("Datasets must be given as a list of strings.")
@@ -1044,7 +1068,7 @@ class AlmaClass(QueryWithLogin):
             data from an ASDM tarball
         """
 
-        if isinstance(urls, six.string_types):
+        if isinstance(urls, str):
             urls = [urls]
         if not isinstance(urls, (list, tuple, np.ndarray)):
             raise TypeError("Datasets must be given as a list of strings.")
@@ -1077,7 +1101,7 @@ class AlmaClass(QueryWithLogin):
                 expanded_files += [x for x in files['access_url'] if
                                    filere.match(x.split('/')[-1])]
             else:
-                tar_files.append(tar_file)
+                tar_files.append(url)
 
         try:
             # get the tar files
@@ -1145,7 +1169,6 @@ class AlmaClass(QueryWithLogin):
         March 2020 - should be removed along with stage_data_prefeb2020
         """
         from ..utils import url_helpers
-        from six import iteritems
         columns = {'mous_uid': [], 'URL': [], 'size': []}
         for entry in data['node_data']:
             # de_type can be useful (e.g., MOUS), but it is not necessarily
@@ -1183,7 +1206,7 @@ class AlmaClass(QueryWithLogin):
 
         columns['size'] = u.Quantity(columns['size'], u.Gbyte)
 
-        tbl = Table([Column(name=k, data=v) for k, v in iteritems(columns)])
+        tbl = Table([Column(name=k, data=v) for k, v in columns.items()])
         return tbl
 
     def get_project_metadata(self, projectid, cache=True):
