@@ -7,14 +7,13 @@ from datetime import datetime
 import os
 from urllib.parse import urlparse
 import re
-from unittest.mock import Mock, patch
+from unittest.mock import Mock, MagicMock, patch
 
 from astropy import coordinates
 from astropy import units as u
 
 from astroquery.utils.commons import ASTROPY_LT_4_1
 from .. import Alma
-from .. import _url_list, _test_url_list
 
 # ALMA tests involving staging take too long, leading to travis timeouts
 # TODO: make this a configuration item
@@ -29,27 +28,23 @@ all_colnames = {'Project code', 'Source name', 'RA', 'Dec', 'Band',
                 'QA2 Status', 'Group ous id', 'Pub'}
 
 
-def get_client():
+@pytest.fixture
+def alma(request):
+    """
+    Returns an alma client class. `--alma-site` pytest option can be used
+    to have the client run against a specific site
+    :param request: pytest request fixture
+    :return: alma client to use in tests
+    """
     alma = Alma()
-    # need this to point alma to a different test site
-    # alma package __init__.py mentions test sites but I don't know how the
-    # mechanism is supposed to be used
-    from .. import core
-    core.ALMA_TAP_PATH = 'obscore'
-    alma.archive_url = 'https://alma.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/'
+    alma_site = request.config.getoption('--alma-site',
+                                         'almascience.org')
+    alma.archive_url = 'https://{}'.format(alma_site)
     return alma
 
 
 @pytest.mark.remote_data
 class TestAlma:
-    def setup_class(cls):
-        pass
-        # new test server
-        # this server seems not to serve a help page?
-        # Alma.archive_url = "https://2016-03.asa-test.alma.cl/aq/"
-        # starting somewhere between Nov 2015 and Jan 2016, the beta server
-        # stopped serving the actual data, making all staging attempts break
-
     @pytest.fixture()
     def temp_dir(self, request):
         my_temp_dir = tempfile.mkdtemp()
@@ -59,8 +54,7 @@ class TestAlma:
         request.addfinalizer(fin)
         return my_temp_dir
 
-    def test_public(self):
-        alma = get_client()
+    def test_public(self, alma):
         results = alma.query(payload=None, public=True, maxrec=100)
         assert len(results) == 100
         for row in results:
@@ -70,8 +64,7 @@ class TestAlma:
         for row in results:
             assert row['data_rights'] == 'Proprietary'
 
-    def test_SgrAstar(self, temp_dir):
-        alma = get_client()
+    def test_SgrAstar(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         result_s = alma.query_object('Sgr A*', legacy_columns=True)
@@ -80,17 +73,14 @@ class TestAlma:
         # "The Brick", g0.253, is in this one
         # assert b'2011.0.00217.S' in result_c['Project code'] # missing cycle 1 data
 
-    def test_docs_example(self, temp_dir):
-        alma = get_client()
+    def test_docs_example(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         rslt = alma.query(payload=dict(obs_creator_name='*Ginsburg*'))
 
         assert 'ADS/JAO.ALMA#2013.1.00269.S' in rslt['obs_publisher_did']
 
-    def test_freq(self):
-        alma = get_client()
-
+    def test_freq(self, alma):
         payload = {'frequency': '85..86'}
         result = alma.query(payload)
         assert len(result) > 0
@@ -103,18 +93,16 @@ class TestAlma:
     @pytest.mark.skipif("SKIP_SLOW",
                         reason="Extremely slow due to limitations of "
                                "the implementation")
-    def test_bands(self):
-        alma = get_client()
+    def test_bands(self, alma):
         payload = {'band_list': ['5', '7']}
         result = alma.query(payload)
         assert len(result) > 0
         for row in result:
             assert ('5' in row['band_list']) or ('7' in row['band_list'])
 
-    def test_equivalent_columns(self):
+    def test_equivalent_columns(self, alma):
         # this test is to ensure that queries using original column names
         # return the same results as the ones that use ObsCore names
-        alma = get_client()
         # original
         result_orig = alma.query(payload={'project_code': '2011.0.00131.S'},
                                  legacy_columns=True)
@@ -126,8 +114,7 @@ class TestAlma:
         for row in result_obscore:
             assert row['Project code'] == '2011.0.00131.S'
 
-    def test_alma_source_name(self):
-        alma = get_client()
+    def test_alma_source_name(self, alma):
         payload = {'source_name_alma': 'GRB021004'}
         result = alma.query(payload)
         assert len(result) > 0
@@ -135,15 +122,13 @@ class TestAlma:
             assert 'GRB021004' == row['target_name']
 
     @pytest.mark.skipif("SKIP_SLOW", reason="Known issue")
-    def test_ra_dec(self):
-        alma = get_client()
+    def test_ra_dec(self, alma):
         payload = {'ra_dec': '181.0192d -0.01928d'}
         result = alma.query(payload)
         assert len(result) > 0
 
     @pytest.mark.skipif("SKIP_SLOW")
-    def test_m83(self, temp_dir, recwarn):
-        alma = get_client()
+    def test_m83(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         m83_data = alma.query_object('M83', science=True, legacy_columns=True)
@@ -170,8 +155,7 @@ class TestAlma:
         #                           ' otherwise you may need to create a fresh astroquery.Alma instance.'))
 
     @pytest.mark.skipif("SKIP_SLOW", reason="Known issue")
-    def test_stage_data(self, temp_dir, recwarn):
-        alma = get_client()
+    def test_stage_data(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         result_s = alma.query_object('Sgr A*', legacy_columns=True)
@@ -201,11 +185,10 @@ class TestAlma:
                 break
         assert found, 'URL to uid___A002_X40d164_X1b3 expected'
 
-    def test_stage_data_listall(self, temp_dir, recwarn):
+    def test_stage_data_listall(self, temp_dir, alma):
         """
         test for expanded capability created in #1683
         """
-        alma = get_client()
         alma.cache_location = temp_dir
 
         uid = 'uid://A001/X12a3/Xe9'
@@ -233,11 +216,10 @@ class TestAlma:
             assert res['mous_uid'] == uid
         assert len(result2) > len(result1)
 
-    def test_stage_data_json(self, temp_dir, recwarn):
+    def test_stage_data_json(self, temp_dir, alma):
         """
         test for json returns
         """
-        alma = get_client()
         alma.cache_location = temp_dir
 
         uid = 'uid://A001/X12a3/Xe9'
@@ -249,9 +231,8 @@ class TestAlma:
             # this no longer works
             alma.stage_data(uid, return_json=True)
 
-    def test_data_proprietary(self):
+    def test_data_proprietary(self, alma):
         # public
-        alma = get_client()
         assert not alma.is_proprietary('uid://A001/X12a3/Xe9')
         IVOA_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"
         now = datetime.utcnow().strftime(IVOA_DATE_FORMAT)[:-3]
@@ -265,8 +246,7 @@ class TestAlma:
         with pytest.raises(AttributeError):
             alma.is_proprietary('uid://NON/EXI/STING')
 
-    def test_data_info(self, temp_dir):
-        alma = get_client()
+    def test_data_info(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         uid = 'uid://A001/X12a3/Xe9'
@@ -304,45 +284,37 @@ class TestAlma:
             'access_url']
         assert comparison.all()
 
-    def test_download_data(self, temp_dir):
+    def test_download_data(self, temp_dir, alma):
         # test only fits files from a program
-        def myrequests(op, file_url, **kwargs):
-            # this is to avoid downloading the actual files
-            if op == 'HEAD':
-                return Mock(headers={'Content-Type': 'fits'})
-            else:
-                return file_url.split('/')[-1]
-        alma = get_client()
         alma.cache_location = temp_dir
 
         uid = 'uid://A001/X12a3/Xe9'
         data_info = alma.get_data_info(uid, expand_tarfiles=True)
         fitsre = re.compile(r'.*\.fits$')
-        alma._request = Mock(side_effect=myrequests)
+        # skip the actual downloading of the file
+        download_mock = MagicMock()
+        # following line require to make alma picklable
+        download_mock.__reduce__ = lambda self: (MagicMock, ())
+        alma._download_file = download_mock
         urls = [x['access_url'] for x in data_info
                 if fitsre.match(x['access_url'])]
         results = alma.download_files(urls, temp_dir)
-        alma._request.assert_called()
+        alma._download_file.call_count == len(results)
         assert len(results) == len(urls)
-        # each url triggers 2 calls: HEAD and GET
-        assert len(urls)*2 == len(alma._request.mock_calls)
 
-    def test_download_and_extract(self, temp_dir):
-        def myrequests(op, file_url, **kwargs):
-            # this is to avoid downloading the actual files
-            if op == 'HEAD':
-                return Mock(headers={'Content-Type': 'fits'})
-            else:
-                return file_url.split('/')[-1]
-        alma = get_client()
+    def test_download_and_extract(self, temp_dir, alma):
         alma.cache_location = temp_dir
-        alma._request = Mock(side_effect=myrequests)
         alma._cycle0_tarfile_content_table = {'ID': ''}
 
         uid = 'uid://A001/X12a3/Xe9'
         data_info = alma.get_data_info(uid, expand_tarfiles=False)
         aux_tar_file = [x for x in data_info['access_url'] if 'auxiliary' in x]
         assert 1 == len(aux_tar_file)
+        download_mock = MagicMock()
+        # following line is required to make alma picklable
+        download_mock.__reduce__ = lambda self: (MagicMock, ())
+        alma._download_file = download_mock
+
         # there are no FITS files in the auxiliary file
         assert not alma.download_and_extract_files(aux_tar_file)
 
@@ -350,7 +322,7 @@ class TestAlma:
         downloaded = alma.download_and_extract_files(aux_tar_file,
                                                      regex=r'.*\.py')
         assert len(downloaded) > 1
-        assert len(downloaded)*2 == len(alma._request.mock_calls)
+        assert download_mock.call_count == len(downloaded)
 
         # ASDM files cannot be expanded.
         asdm_url = [x for x in data_info['access_url'] if 'asdm' in x][0]
@@ -369,15 +341,13 @@ class TestAlma:
             with patch('astroquery.alma.core.os.remove') as delete_mock:
                 downloaded_asdm = alma.download_and_extract_files(
                     [asdm_url], include_asdm=True, regex=r'.*\.py')
-        delete_mock.assert_called_once_with(asdm_url.split('/')[-1])
+        delete_mock.assert_called_once_with(
+            'cache_path/' + asdm_url.split('/')[-1])
         assert downloaded_asdm == [os.path.join(temp_dir, 'foo.py')]
 
     @pytest.mark.skipif("SKIP_SLOW", reason="Known issue")
-    def test_doc_example(self, temp_dir):
-        alma = get_client()
+    def test_doc_example(self, temp_dir, alma):
         alma.cache_location = temp_dir
-        alma2 = get_client()
-        alma2.cache_location = temp_dir
         m83_data = alma.query_object('M83', legacy_columns=True)
         # the order can apparently sometimes change
         # These column names change too often to keep testing.
@@ -404,15 +374,14 @@ class TestAlma:
         totalsize_mous1 = mous1['size'].sum() * u.Unit(mous1['size'].unit)
         assert (totalsize_mous1.to(u.B) > 1.9*u.GB)
 
-        mous = alma2.stage_data('uid://A002/X3216af/X31')
+        mous = alma.stage_data('uid://A002/X3216af/X31')
         totalsize_mous = mous['size'].sum() * u.Unit(mous['size'].unit)
         # More recent ALMA request responses do not include any information
         # about file size, so we have to allow for the possibility that all
         # file sizes are replaced with -1
         assert (totalsize_mous.to(u.GB).value > 52)
 
-    def test_query(self, temp_dir):
-        alma = get_client()
+    def test_query(self, temp_dir, alma):
         alma.cache_location = temp_dir
 
         result = alma.query(payload={'start_date': '<11-11-2011'},
@@ -434,9 +403,8 @@ class TestAlma:
         # assert len(result) == 1
 
     @pytest.mark.skipif("SKIP_SLOW", reason="ra dec search known issue")
-    def test_misc(self):
+    def test_misc(self, alma):
         # miscellaneous set of common tests
-        alma = get_client()
         #
         # alma.query_region(coordinate=orionkl_coords, radius=4 * u.arcmin,
         #                  public=False, science=False)
@@ -498,9 +466,8 @@ class TestAlma:
             assert 'ginsburg' in row['obs_creator_name'].lower()
 
     @pytest.mark.skipif("SKIP_SLOW")
-    def test_user(self):
+    def test_user(self, alma):
         # miscellaneous set of tests from current users
-        alma = get_client()
         rslt = alma.query({'band_list': [6], 'project_code': '2012.1.*'},
                           legacy_columns=True)
         for row in rslt:
@@ -512,9 +479,8 @@ class TestAlma:
     @pytest.mark.xfail
     @pytest.mark.bigdata
     @pytest.mark.skipif("SKIP_SLOW")
-    def test_cycle1(self, temp_dir):
+    def test_cycle1(self, temp_dir, alma):
         # About 500 MB
-        alma = get_client()
         alma.cache_location = temp_dir
         target = 'NGC4945'
         project_code = '2012.1.00912.S'
@@ -560,10 +526,8 @@ class TestAlma:
 
     @pytest.mark.skipif("SKIP_SLOW")
     @pytest.mark.skip("Not working anymore")
-    def test_cycle0(self, temp_dir):
+    def test_cycle0(self, temp_dir, alma):
         # About 20 MB
-
-        alma = get_client()
         alma.cache_location = temp_dir
 
         target = 'NGC4945'
@@ -597,9 +561,7 @@ class TestAlma:
         # There are 10 small files, but only 8 unique
         assert len(data) == 8
 
-    def test_keywords(self, temp_dir):
-
-        alma = get_client()
+    def test_keywords(self, temp_dir, alma):
 
         alma.help_tap()
         result = alma.query_tap(
@@ -613,8 +575,7 @@ class TestAlma:
 
 
 @pytest.mark.remote_data
-def test_project_metadata():
-    alma = get_client()
+def test_project_metadata(alma):
     metadata = alma.get_project_metadata('2013.1.00269.S')
     assert metadata == ['Sgr B2, a high-mass molecular cloud in our Galaxy\'s '
                         'Central Molecular Zone, is the most extreme site of '
@@ -643,11 +604,9 @@ def test_project_metadata():
 
 
 @pytest.mark.remote_data
-@pytest.mark.parametrize('dataarchive_url', _test_url_list)
 @pytest.mark.skip('Not working for now - Investigating')
-def test_staging_postfeb2020(dataarchive_url):
+def test_staging_postfeb2020(alma):
 
-    alma = get_client()
     tbl = alma.stage_data('uid://A001/X121/X4ba')
 
     assert 'mous_uid' in tbl.colnames
@@ -656,11 +615,8 @@ def test_staging_postfeb2020(dataarchive_url):
 
 
 @pytest.mark.remote_data
-@pytest.mark.parametrize('dataarchive_url', _url_list)
 @pytest.mark.skip('Not working for now - Investigating')
-def test_staging_uptofeb2020(dataarchive_url):
-
-    alma = get_client()
+def test_staging_uptofeb2020(alma):
     tbl = alma.stage_data('uid://A001/X121/X4ba')
 
     assert 'mous_uid' in tbl.colnames
@@ -671,9 +627,31 @@ def test_staging_uptofeb2020(dataarchive_url):
 
 
 @pytest.mark.remote_data
-@pytest.mark.parametrize('dataarchive_url', _test_url_list)
-def test_staging_stacking(dataarchive_url):
-    alma = get_client()
-
+def test_staging_stacking(alma):
     alma.stage_data(['uid://A001/X13d5/X1d', 'uid://A002/X3216af/X31',
                      'uid://A001/X12a3/X240'])
+
+
+@pytest.mark.remote_data
+@pytest.mark.skipif("SKIP_SLOW", reason="Huge data file download")
+def test_big_download_regression(alma):
+    """
+    Regression test for #2020/#2021 - this download fails if logging tries to
+    load the whole data file into memory.
+    """
+    result = alma.query({'project_code': '2013.1.01365.S'})
+    uids = np.unique(result['member_ous_uid'])
+    files = alma.get_data_info(uids)
+
+    # we may need to change the cache dir for this to work on testing machines?
+    # savedir='/big/data/path/'
+    # Alma.cache_dir=savedir
+
+    # this is a big one that fails
+    alma.download_files([files['access_url'][3]])
+
+
+@pytest.mark.remote_data
+def test_download_html_file():
+    result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'])
+    assert result
